%radioactive_decay_analysis
% Analysis of Iodine-131 decay rate vs time data.
%
% LAST UPDATED by Andy French June 2019

function radioactive_decay_analysis

%Estimated background rate /Bq
B = 123;

%Ignore data for line of best fit after this time (days)
t_ignore = 28.75;

%Fontsize for graphs
fsize = 18;

%

%Ingest Excel file of activity vs time
[num,txt,raw] = xlsread( 'iodine-131 activity.xls' );

%Extract vectors for time /days and activity /Bq
t = num(:,1); A = num(:,2);

%Plot activity vs time
fig1 = figure('color',[1 1 1],'name','radioactive decay curve');
plot(t,A,'+');
xlabel('time /days','fontsize',fsize);
ylabel('decay rate /Bq ','fontsize',fsize);
set(gca,'fontsize',fsize);
grid on; ylim([0,max(A)]);

%Overlay background level
xlimits = get( gca, 'xlim' ); hold on; plot( xlimits,[B,B],'r-','linewidth',2 );

%Plot log( A - B ) vs t
x = A - B;
y = log(x); y( x<=0 ) = NaN;
fig2 = figure('color',[1 1 1],'name','radioactive decay curve');
plot(t,y,'+');
xlabel('time /days','fontsize',fsize);
ylabel('log( decay rate - background ) ','fontsize',fsize);
set(gca,'fontsize',fsize); grid on;

%Perform line of best fit and hence determine theoretical 
%initial activity /Bq and half life /days
ok = (t <= t_ignore );
[yfit,xfit,r,m,c,dm,dc,yupper,ylower,s] = bestfitc( t(ok),y(ok) );
hold on; plot( xfit, yfit, 'r-','linewidth',2 );
half_life = -log(2)/m; A0 = round( exp(c) );
half_life_upper = log(2)/(-m-dm);
half_life_lower = log(2)/(-m+dm);
half_life_error = (half_life_upper-half_life_lower)/2;
title(['A_0 = ',num2str(A0),'Bq, half life = ',num2str(half_life,4),...
    ' +/- ',num2str(half_life_error,3),' days'],'fontsize',fsize);
legend({'Data','Best fit'},'fontsize',fsize,'location','northeast');

%Print analysis graph
print( fig2, 'radioactivity analysis log graph.png','-dpng','-r300' );
close(fig2);

%Overlay model curve on original data
figure(fig1);
A_model = A0*2.^(-t/half_life) + B;
hold on; plot( t, A_model, 'g-','linewidth',2);
title( {
    ['Activity of Iodine-131. Background = ',num2str(B),'Bq'],...
    ['A_0 = ',num2str(A0),'Bq, half life = ',num2str(half_life,4),...
    ' +/- ',num2str(half_life_error,3),' days']
    },'fontsize',fsize);
legend({'Data','Background','Model'},'fontsize',fsize,'location','northeast');

%Print data display graph
print( fig1, 'radioactivity analysis graph.png','-dpng','-r300' );
close(fig1);

%%

%Line of best fit function yfit = m*x +c, with product moment correlation
%coefficient r
function [yfit,xfit,r,m,c,dm,dc,yupper,ylower,s] = bestfitc(x,y)

%Find any x or y values that are NaN or Inf
ignore = isnan(abs(x)) | isnan(abs(y)) | isinf(abs(x)) | isinf(abs(y));
x(ignore) = []; y(ignore) = [];
n = length(x);

%Compute line of best fit
xbar = mean(x); ybar = mean(y); xybar = mean(x.*y);
xxbar = mean(x.^2 ); yybar = mean(y.^2 );
Vx = xxbar - xbar^2; Vy = yybar - ybar^2;
COVxy = xybar - xbar*ybar;
m = COVxy/Vx; c = ybar - m*xbar; r = COVxy/sqrt( Vx*Vy );
[x,i] = sort(x); y = y(i);
yfit = m*x + c; xfit = x;

%Compute errors in gradient m and intercept c
s = sqrt( (1/(n-2))*sum( (y - yfit).^2 ) );
dm = s/sqrt(n*Vx);
dc = ( s/sqrt(n) )*sqrt( 1 + (xbar^2)/Vx );

%Determine envelope lines
yupper = (m+dm)*x + c + dc;
ylower = (m-dm)*x + c - dc;

%End of code